@@ -14,7 +14,7 @@ use core::sync::atomic::{
 };
 use core::{fmt, marker};
 
-use crate::de::{Decode, Decoder, PairDecoder};
+use crate::de::{Decode, Decoder, PairDecoder, ReferenceVisitor};
 use crate::en::{Encode, Encoder, VariantEncoder};
 use crate::error::Error;
 
@@ -249,7 +249,7 @@ impl Encode for str {
     where
         E: Encoder,
     {
-        encoder.encode_str(self)
+        encoder.encode_string(self)
     }
 }
 
@@ -259,7 +259,28 @@ impl<'de> Decode<'de> for &'de str {
     where
         D: Decoder<'de>,
     {
-        decoder.decode_str()
+        return decoder.decode_string(Visitor(marker::PhantomData));
+
+        struct Visitor<E>(marker::PhantomData<E>);
+
+        impl<'de, E> ReferenceVisitor<'de> for Visitor<E>
+        where
+            E: Error,
+        {
+            type Target = str;
+            type Ok = &'de str;
+            type Error = E;
+
+            #[inline]
+            fn expected(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                write!(f, "expected exact string reference")
+            }
+
+            #[inline]
+            fn visit_ref(self, string: &'de str) -> Result<Self::Ok, Self::Error> {
+                Ok(string)
+            }
+        }
     }
 }
 
@@ -278,7 +299,28 @@ impl<'de> Decode<'de> for &'de [u8] {
     where
         D: Decoder<'de>,
     {
-        decoder.decode_bytes()
+        return decoder.decode_bytes(Visitor(marker::PhantomData));
+
+        struct Visitor<E>(marker::PhantomData<E>);
+
+        impl<'de, E> ReferenceVisitor<'de> for Visitor<E>
+        where
+            E: Error,
+        {
+            type Target = [u8];
+            type Ok = &'de [u8];
+            type Error = E;
+
+            #[inline]
+            fn expected(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                write!(f, "expected exact bytes reference")
+            }
+
+            #[inline]
+            fn visit_ref(self, bytes: &'de [u8]) -> Result<Self::Ok, Self::Error> {
+                Ok(bytes)
+            }
+        }
     }
 }
 
